defmodule DecentralisedBookIndexWeb.BookLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      Listing Books
      <:actions>
        <.link patch={~p"/books/new"}>
          <.button>New Book</.button>
        </.link>
      </:actions>
    </.header>

    <.table
      id="books"
      rows={@streams.books}
      row_click={fn {_id, book} -> JS.navigate(~p"/books/#{book}") end}
    >
      <:col :let={{_id, book}} label="Id">{book.id}</:col>

      <:col :let={{_id, book}} label="Title">{book.title}</:col>

      <:col :let={{_id, book}} label="Description">{book.description}</:col>

      <:col :let={{_id, book}} label="Published">{book.published}</:col>

      <:col :let={{_id, book}} label="Language">{book.language}</:col>

      <:col :let={{_id, book}} label="Format">{book.format}</:col>

      <:col :let={{_id, book}} label="Page count">{book.page_count}</:col>

      <:col :let={{_id, book}} label="Cover image url">{book.cover_image_url}</:col>

      <:action :let={{_id, book}}>
        <div class="sr-only">
          <.link navigate={~p"/books/#{book}"}>Show</.link>
        </div>

        <.link patch={~p"/books/#{book}/edit"}>Edit</.link>
      </:action>

      <:action :let={{id, book}}>
        <.link
          phx-click={JS.push("delete", value: %{id: book.id}) |> hide("##{id}")}
          data-confirm="Are you sure?"
        >
          Delete
        </.link>
      </:action>
    </.table>

    <%= if Enum.empty?(@page.results) do %>
      <div class="flex justify-center ">
        <div>
          <p class="text-xl font-semibold py-5">
            No Books
          </p>
        </div>
      </div>
    <% else %>
      <div class="flex flex-wrap flex-[3_1_auto]">
        <%= for book <- @page.results do %>
          <.book_card book={book} />
        <% end %>
      </div>

      <.pagination
        endpoint={~p"/books"}
        page={@page}
        page_params={@page_params}
        params={@params}
      />
    <% end %>

    <.modal :if={@live_action in [:new, :edit]} id="book-modal" show on_cancel={JS.patch(~p"/books")}>
      <.live_component
        module={DecentralisedBookIndexWeb.BookLive.FormComponent}
        id={(@book && @book.id) || :new}
        title={@page_title}
        current_user={@current_user}
        action={@live_action}
        book={@book}
        patch={~p"/books"}
      />
    </.modal>
    """
  end

  @impl true
  def mount(params, _session, socket) do
    search_query = Map.get(params, "query", "")
    sort_by = Map.get(params, "sort_by", "name") |> validate_sort_by()
    page_params = AshPhoenix.LiveView.page_from_params(params, 10)

    page = Metadata.search_book!(
      search_query,
      page: page_params ++ [count: true],
      actor: socket.assigns[:current_user]
    )

    socket =
      socket
      |> stream(
        :books,
        Ash.read!(DecentralisedBookIndex.Metadata.Book, actor: socket.assigns[:current_user])
      )
      |> assign(:sort_by, sort_by)
      |> assign(:search_query, search_query)
      |> assign(:select_options, sort_options())
      |> assign(:page_params, page_params)
      |> assign(:page, page)
      |> assign(:params, params)
      |> assign_new(:current_user, fn -> nil end)

    {:ok, socket}
  end

  @impl true
  def handle_params(params, _url, socket) do
    {:noreply, apply_action(socket, socket.assigns.live_action, params)}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Book")
    |> assign(
      :book,
      Ash.get!(DecentralisedBookIndex.Metadata.Book, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Book")
    |> assign(:book, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Books")
    |> assign(:book, nil)
  end

  @impl true
  def handle_info({DecentralisedBookIndexWeb.BookLive.FormComponent, {:saved, book}}, socket) do
    {:noreply, stream_insert(socket, :books, book)}
  end

  @impl true
  def handle_event("delete", %{"id" => id}, socket) do
    book = Ash.get!(DecentralisedBookIndex.Metadata.Book, id, actor: socket.assigns.current_user)
    Ash.destroy!(book, actor: socket.assigns.current_user)

    {:noreply, stream_delete(socket, :books, book)}
  end

  defp sort_options do
    [
      {"Sort by title", "title"},
      {"Sort by recently updated", "-updated_at"},
      {"Sort by recently added", "-inserted_at"},
    ]
  end

  def validate_sort_by(key) do
    valid_keys = Enum.map(sort_options(), &elem(&1, 1))

    if key in valid_keys do
      key
    else
      List.first(valid_keys)
    end
  end
end
